#include "metacube.h"


/** FUNCTION "error" **/

void error (char const *restrict format, ...)
{
  va_list vlist;
  char *buffer;
  int bufsz;

  va_start (vlist, format);
  bufsz = vsnprintf (NULL, 0, format, vlist) + 1;
  buffer = malloc (bufsz);
  vsnprintf (buffer, bufsz, format, vlist);
  va_end (vlist);
  ffprintf (2, stderr, logstream, "ERROR - %s", buffer);
  fprintf (stderr, "Program aborted.\n");
  free (buffer);
  exit (EXIT_FAILURE);
}


/** FUNCTION "ffprintf" **/

void ffprintf (int z, ...)
{
  va_list vlist, thevlist;
  FILE **streams;
  char const *format;
  int i;

  streams = malloc (z * sizeof(FILE *));
  va_start (vlist, z);
  for (i = 0; i < z; ++i)
    {
      streams[i] = va_arg (vlist, FILE *);
    }
  format = va_arg (vlist, char const *);
  for (i = 0; i < z; ++i)
    {
      if (streams[i])
        {
          va_copy (thevlist, vlist);
          vfprintf (streams[i], format, thevlist);
          va_end (thevlist);
        }
    }
  va_end (vlist);
  free (streams);
}


/** FUNCTION "fopen_ctrl" **/

FILE *fopen_ctrl (char const *restrict filename, char const *restrict mode)
{
  FILE *stream;

  stream = fopen (filename, mode);
  if (!stream)
    {
      error ("I could not open file \"%s\".\n", filename);
    }
  return stream;
}


/** FUNCTION "strncat_0" **/

void strncat_0 (char *restrict dest, const char *restrict src, size_t count)
{
  int destlen;

  destlen = strlen (dest);
  strncpy_0 (dest + destlen, src, count - destlen);
}


/** FUNCTION "strncpy_0" **/

void strncpy_0 (char *restrict dest, const char *restrict src, size_t count)
{
  strncpy (dest, src, count - 1);
  dest [count - 1] = '\0';
}


/** FUNCTION "vectmult" **/

struct double3 vectmult (double a, struct double3 v)
{
  v.x *= a;
  v.y *= a;
  v.z *= a;
  return v;
}


/** FUNCTION "vectsum" */

struct double3 vectsum (struct double3 v, struct double3 w)
{
  v.x += w.x;
  v.y += w.y;
  v.z += w.z;
  return v;
}


/** FUNCTION "warning" **/

void warning (char const *restrict format, ...)
{
  va_list vlist;
  char *buffer;
  int bufsz;

  va_start (vlist, format);
  bufsz = vsnprintf (NULL, 0, format, vlist) + 1;
  buffer = malloc (bufsz);
  vsnprintf (buffer, bufsz, format, vlist);
  va_end (vlist);
  ffprintf (2, stderr, logstream, "WARNING - %s", buffer);
  fprintf (stderr, "Continuing processing...\n");
  free (buffer);
}
