/** This is the header for the auxiliary functions of program METACUBE **/

#ifndef AUXFUNCTIONS_H_INCLUDED
#define AUXFUNCTIONS_H_INCLUDED


/** FUNCTIONAL MACROS **/

/* Macro "METAMALLOC" allocates memory for a meta-array "array" (of any type of the form (type ***)) of size "nnn" (of type (struct int3)) */

#define METAMALLOC(array,nnn) array = malloc (nnn.x * sizeof(*array)); for (int x = 0; x < nnn.x; ++x) {array[x] = malloc (nnn.y * sizeof(**array)); for (int y = 0; y < nnn.y; ++y) {array[x][y] = malloc (nnn.z * sizeof(***array));}}

/* Macro "METAFREE" frees a meta-array "array" (of any type of the form (type ***)) of size "nnn" (of type (struct int3)) */

#define METAFREE(nnn,array) for (int x = 0; x < nnn.x; ++x) {for (int y = 0; y < nnn.y; ++y) {free (array[x][y]);} free (array[x]);} free (array)

/* Macro "METAFOR" makes variables "x", "y" and "z" (of type int) run along all triples of integers less than "nnn" (of type (struct int3)) */

#define METAFOR(x,y,z,nnn) for (x = 0; x < nnn.x; ++x) {for (y = 0; y < nnn.y; ++y) {for (z = 0; z < nnn.z; ++z)

/* Macro "ENDMETAFOR" is used to end a METAFOR loop */

#define ENDMETAFOR }}


/** GLOBAL VARIABLES **/

FILE *logstream; // Stream on which the log is written


/** FUNCTION PROTOTYPES **/

/* Function "error" emits an error message on both "stderr" and "logstream", that message given by "format" and the extra arguments; then the program exits with status EXIT_FAILURE */

void error (char const *restrict format, ...);

/* Function "ffprintf" makes the same work as the standard "fprintf", except that it prints its message onto several streams, whose number is indicated by "z" */

void ffprintf (int z, ...);

/* Function "fopen_ctrl" makes the same work as the standard "fopen", except that in case of failure it emits an error and exits the program */

FILE *fopen_ctrl (char const *restrict filename, char const *restrict mode);

/* Function "strncat_0" makes the same work as the standard "strncat", except that here "count" is not the maximum number of characters appended, but the maximum size the resulting string (including terminating \0) */

void strncat_0 (char *restrict dest, const char *restrict src, size_t count);

/* Function "strncpy_0" makes the same work as the standard "strncpy", except that it ensures that the copied string is null-terminated */

void strncpy_0 (char *restrict dest, const char *restrict src, size_t count);

/* Function "vectmult" computes a.v for a vector "v" and a scalar "a" */

struct double3 vectmult (double a, struct double3 v);

/* Function "vectsum" computes v+w for two vectors "v" and "w" */

struct double3 vectsum (struct double3 v, struct double3 w);

/* Function "warning" emits a warning message on both "stderr" and "logstream", that message given by "format" and the extra arguments; then processing continues */

void warning (char const *restrict format, ...);

#endif // AUXFUNCTIONS_H_INCLUDED
