#include "metacube.h"

void recube (double ****p_density, struct int3 nnn)
{


  /** VARIABLE DECLARATIONS **/

  /* Constants */

  double const kernel[3] = {1. / 8, 3. / 4, 1. / 8}; // Convolution kernel along any coordinate

  /* Variables */

  double ***densitynew; // New density data
  int x, y, z; // Indices for locating a node in the grid
  int xi, yi, zi; // Indices for initial positions
  int dx, dy, dz; // Indices for position differences


  /** FILLING "densitynew" **/

  /* Initialization */

  METAMALLOC (densitynew, nnn);
  METAFOR (x, y, z, nnn)
  {
    densitynew[x][y][z] = 0;

    /* Scanning the neighbors of node (x,y,z) in the input grid */

    for (dx = -1; dx <= 1; ++dx)
      {
        for (dy = -1; dy <= 1; ++dy)
          {
            for (dz = -1; dz <= 1; ++dz)
              {
                xi = x + dx;
                yi = y + dx;
                zi = z + dz;
                if (xi < 0) // If (xi,yi,zi) is outside the grid, we have to replace it by the closest valid node
                  {
                    xi = 0;
                  }
                else if (xi >= nnn.x)
                  {
                    xi = nnn.x - 1;
                  }
                if (yi < 0)
                  {
                    yi = 0;
                  }
                else if (yi >= nnn.y)
                  {
                    yi = nnn.y - 1;
                  }
                if (zi < 0)
                  {
                    zi = 0;
                  }
                else if (zi >= nnn.z)
                  {
                    zi = nnn.z - 1;
                  }
                densitynew[x][y][z] += kernel[dx + 1] * kernel[dy + 1] * kernel[dz + 1] * (*p_density)[xi][yi][zi];
              }
          }
      }
  }
  ENDMETAFOR


  /** REDIRECTING "*p_density" TO "densitynew" **/

  METAFREE (nnn, (*p_density));
  *p_density = densitynew;
}
